/*
  adc.c - Demonstrates how to read the 8 ADC channels
              by Micromint Support <support@micromint.com>   May-2009

  Copyright (c) 2008-2010 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "inc/hw_adc.h"
#include "inc/hw_gpio.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/adc.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "utils/uartstdio.h"
#include "ftoa.h"

#define LED_PIN          GPIO_PIN_1
#define LED_REG          HWREG(GPIO_PORTE_BASE + (GPIO_O_DATA + (LED_PIN << 2)))
#define NUM_ADCS         8
#define ADC_CTL_CH(c)    (ADC_CTL_CH0 + c)

/*
  Handler for SysTick interrupt
*/
void SysTickHandler(void)
{
    unsigned long i, adc_sample[8], temperature[1];
    char temp[8];
    float celsius, fahrenheit;

    LED_REG ^= LED_PIN; /* Toggle LED */
    /* Trigger the sample sequence 0 */
    ADCProcessorTrigger(ADC_BASE, 0);
    /* Wait until the sample sequence has completed. */
    while(!ADCIntStatus(ADC_BASE, 0, false))
    {
    }

    /* Clear the interrupt flag */
    HWREG(ADC_BASE + ADC_O_ISC) |= ADC_ISC_IN0;
    ADCSequenceDataGet(ADC_BASE, 0, adc_sample);
    /* Trigger the sample sequence 3 */
    ADCProcessorTrigger(ADC_BASE, 3);
    /* Wait until the sample sequence has completed */
    while(!ADCIntStatus(ADC_BASE, 3, false))
    {
    }

    /* Clear the interrupt flag */
    HWREG(ADC_BASE + ADC_O_ISC) |= ADC_ISC_IN3;
    ADCSequenceDataGet(ADC_BASE, 3, temperature);

    /* Display the readings */
    for (i = 0; i < NUM_ADCS; i++)
    {
        ftoa(temp,3,(3.0/1024)*adc_sample[i]);
        UARTprintf("Channel %d = %d = %sV\n",i, adc_sample[i], temp);
    }
    celsius = (147.5-(75*((3.0/1024)*temperature[0])));
    fahrenheit = (celsius * 1.8)+32.0;
    ftoa(temp,1,celsius);
    UARTprintf("Core Temperature = %d = %sC ",temperature[0],temp);
    ftoa(temp,1,fahrenheit);
    UARTprintf("= %sF\n\n",temp);
}

/*
  Main function to adc example
*/
int main(void)
{
    unsigned long i;

    /* Adjust LDO to 2.75V according to REV A2 errata when using PLL */
    SysCtlLDOSet(SYSCTL_LDO_2_75V);
    /* Set the clock to run directly from the crystal */
    SysCtlClockSet(SYSCTL_SYSDIV_1 | SYSCTL_USE_OSC | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);

    /* Set the clock to run at 50 MHz using the PLL */
    //SysCtlClockSet(SYSCTL_SYSDIV_4| SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
    //               SYSCTL_XTAL_8MHZ);

    /* Enable peripherals and set GPIO pins */
    /* --- Enable LED --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    HWREG(GPIO_PORTE_BASE + GPIO_O_DIR) |= LED_PIN;
    HWREG(GPIO_PORTE_BASE + GPIO_O_DEN) |= LED_PIN;
    /* --- Enable ADC --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC);
    SysCtlADCSpeedSet(SYSCTL_ADCSPEED_500KSPS);
    /* Enable sequencer 0 for all 8 channels oaf ADC */
    ADCSequenceConfigure(ADC_BASE, 0, ADC_TRIGGER_PROCESSOR, 0);
    /* Enable sequencer 3 for temperature sensor */
    ADCSequenceConfigure(ADC_BASE, 3, ADC_TRIGGER_PROCESSOR, 1);
    /* Enable all ADC channels */
    for (i = 0; i < (NUM_ADCS - 1); i++)
    {
        ADCSequenceStepConfigure(ADC_BASE, 0, i, ADC_CTL_CH(i));
    }
    ADCSequenceStepConfigure(ADC_BASE, 0, (NUM_ADCS - 1),
                             ADC_CTL_IE | ADC_CTL_END | ADC_CTL_CH(NUM_ADCS - 1));
    /* Enable the temperature sensor */
    ADCSequenceStepConfigure(ADC_BASE, 3, 0,
                             ADC_CTL_IE | ADC_CTL_END | ADC_CTL_TS);
    ADCSequenceEnable(ADC_BASE, 0);
    ADCSequenceEnable(ADC_BASE, 3);
    /* --- Enable UART --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);

    /* Configure SysTick to interrupt twice per second (500 ms)  */
    SysTickPeriodSet(SysCtlClockGet()/2);
    SysTickEnable();
    IntRegister(FAULT_SYSTICK, SysTickHandler);
    SysTickIntEnable();

    /* Test ADC */
    UARTprintf("ADC Example\n");
    /* Loop while waiting for interrupts */
    while(1) {}
}
