//*****************************************************************************
//
// enet_io.c - I/O control via a web server.
//
// Copyright (c) 2007-2010 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of the Micromint Eagle Firmware Package.
//
//*****************************************************************************

#include <string.h>
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/debug.h"
#include "driverlib/ethernet.h"
#include "driverlib/flash.h"
#include "driverlib/gpio.h"
#include "driverlib/adc.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "utils/lwiplib.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "httpserver_raw/httpd.h"
#include "io.h"
#include "cgifuncs.h"

#define IP_ADDR(a,b,c,d) (((u32_t)((a) & 0xff) << 24) | \
                          ((u32_t)((b) & 0xff) << 16) | \
                          ((u32_t)((c) & 0xff) << 8) | \
                           (u32_t)((d) & 0xff))

#if defined(ewarm)
#define g_pfnVectors __vector_table
#endif
//extern void (*g_pfnVectors[NUM_INTERRUPTS])(void);

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>Ethernet-based I/O Control (enet_io)</h1>
//!
//! This example application demonstrates web-based I/O control using the
//! Stellaris Ethernet controller and the lwIP TCP/IP Stack.  DHCP is used to
//! obtain an Ethernet address.  If DHCP times out without obtaining an
//! address, a static IP address will be chosen using AutoIP.  The address that
//! is selected will be shown on the serial port allowing you to access the
//! internal web pages served by the application via your normal web browser.
//!
//! Two different methods of controlling board peripherals via web pages are
//! illustrated via pages labelled "IO Control Demo 1" and "IO Control Demo 2"
//! in the navigation menu on the left of the application's home page.
//!
//! "IO Control Demo 1" uses JavaScript running in the web browser to send
//! HTTP requests for particular special URLs.  These special URLs are
//! intercepted in the file system support layer (lmi_fs.c) and used to
//! control the LED and one GPIO port.  Responses generated by the board are
//! returned to the browser and inserted into the page HTML dynamically by
//! more JavaScript code.
//!
//! "IO Control Demo 2" uses standard HTML forms to pass parameters to CGI
//! (Common Gateway Interface) handlers running on the board.  These handlers
//! process the form data and control the LED and GPIO port as requested before
//! sending a response page (in this case, the original form) back to the
//! browser.  The application registers the names and handlers for each of its
//! CGIs with the HTTPD server during initialization and the server calls
//! these handlers after parsing URL parameters each time one of the CGI URLs
//! is requested.
//!
//! Information on the state of the various controls in the second demo is
//! inserted into the served HTML using SSI (Server Side Include) tags which
//! are parsed by the HTTPD server in the application.  As with the CGI
//! handlers, the application registers its list of SSI tags and a handler
//! function with the web server during initialization and this handler is
//! called whenever any registered tag is found in a .shtml, .ssi or .shtm
//! file being served to the browser.
//!
//! Note that the web server used by this example has been modified from the
//! example shipped with the basic lwIP package.  Additions include SSI and
//! CGI support along with the ability to have the server automatically insert
//! the HTTP headers rather than having these built in to the files in the
//! file system image.
//!
//! For additional details on lwIP, refer to the lwIP web page at:
//! http://www.sics.se/~adam/lwip/
//
//*****************************************************************************

//*****************************************************************************
//
// Defines for setting up the system clock.
//
//*****************************************************************************
#define SYSTICKHZ               100
#define SYSTICKMS               (1000 / SYSTICKHZ)
#define SYSTICKUS               (1000000 / SYSTICKHZ)
#define SYSTICKNS               (1000000000 / SYSTICKHZ)

//*****************************************************************************
//
// A set of flags.  The flag bits are defined as follows:
//
//     0 -> An indicator that a SysTick interrupt has occurred.
//
//*****************************************************************************
#define FLAG_SYSTICK            0
static volatile unsigned long g_ulFlags;

//*****************************************************************************
//
// External Application references.
//
//*****************************************************************************
extern void httpd_init(void);

//*****************************************************************************
//
// SSI tag indices for each entry in the g_pcSSITags array.
//
//*****************************************************************************
#define SSI_INDEX_PE0STATE    0
#define SSI_INDEX_PE1STATE    1
#define SSI_INDEX_PE2STATE    2
#define SSI_INDEX_PE3STATE    3
#define SSI_INDEX_PC4STATE    4
#define SSI_INDEX_PC5STATE    5
#define SSI_INDEX_PC6STATE    6
#define SSI_INDEX_PC7STATE    7
#define SSI_INDEX_ADC0VALUE   8
#define SSI_INDEX_ADC1VALUE   9
#define SSI_INDEX_ADC2VALUE   10
#define SSI_INDEX_ADC3VALUE   11
#define SSI_INDEX_ADC4VALUE   12
#define SSI_INDEX_ADC5VALUE   13
#define SSI_INDEX_ADC6VALUE   14
#define SSI_INDEX_ADC7VALUE   15
#define SSI_INDEX_FORMVARS    16


//*****************************************************************************
//
// This array holds all the strings that are to be recognized as SSI tag
// names by the HTTPD server.  The server will call SSIHandler to request a
// replacement string whenever the pattern <!--#tagname--> (where tagname
// appears in the following array) is found in ".ssi", ".shtml" or ".shtm"
// files that it serves.
//
//*****************************************************************************
static const char *g_pcConfigSSITags[] =
{
    "PE0txt",        // SSI_INDEX_PC4STATE
    "PE1txt",        // SSI_INDEX_PE3STATE
    "PE2txt",        // SSI_INDEX_PE2STATE
    "PE3txt",        // SSI_INDEX_PE1STATE
    "PC4txt",        // SSI_INDEX_PE0STATE
    "PC5txt",        // SSI_INDEX_PC7STATE
    "PC6txt",        // SSI_INDEX_PC6STATE
    "PC7txt",        // SSI_INDEX_PC5STATE
    "ADC0val",       // SSI_INDEX_ADC0VALUE
    "ADC1val",       // SSI_INDEX_ADC1VALUE
    "ADC2val",       // SSI_INDEX_ADC2VALUE
    "ADC3val",       // SSI_INDEX_ADC3VALUE
    "ADC4val",       // SSI_INDEX_ADC4VALUE
    "ADC5val",       // SSI_INDEX_ADC5VALUE
    "ADC6val",       // SSI_INDEX_ADC6VALUE
    "ADC7val",       // SSI_INDEX_ADC7VALUE
    "FormVars",      // SSI_INDEX_FORMVARS
};

//*****************************************************************************
//
//! The number of individual SSI tags that the HTTPD server can expect to
//! find in our configuration pages.
//
//*****************************************************************************
#define NUM_CONFIG_SSI_TAGS     (sizeof(g_pcConfigSSITags) / sizeof (char *))

//*****************************************************************************
//
//! Prototypes for the various CGI handler functions.
//
//*****************************************************************************
static char *ControlCGIHandler(int iIndex, int iNumParams, char *pcParam[],
                              char *pcValue[]);
static char *SetTextCGIHandler(int iIndex, int iNumParams, char *pcParam[],
                              char *pcValue[]);
static char *AdcCGIHandler(int iIndex, int iNumParams, char *pcParam[],
                              char *pcValue[]);

//*****************************************************************************
//
//! Prototype for the main handler used to process server-side-includes for the
//! application's web-based configuration screens.
//
//*****************************************************************************
static int SSIHandler(int iIndex, char *pcInsert, int iInsertLen);

//*****************************************************************************
//
// CGI URI indices for each entry in the g_psConfigCGIURIs array.
//
//*****************************************************************************
#define CGI_INDEX_CONTROL       0
#define CGI_INDEX_TEXT          1
#define CGI_INDEX_ADC           2

//*****************************************************************************
//
//! This array is passed to the HTTPD server to inform it of special URIs
//! that are treated as common gateway interface (CGI) scripts.  Each URI name
//! is defined along with a pointer to the function which is to be called to
//! process it.
//
//*****************************************************************************
static const tCGI g_psConfigCGIURIs[] =
{
    { "/iocontrol.cgi", ControlCGIHandler },      // CGI_INDEX_CONTROL
    { "/settxt.cgi", SetTextCGIHandler },         // CGI_INDEX_TEXT
    { "/adcinput.cgi", AdcCGIHandler }            // CGI_INDEX_ADC
};

//*****************************************************************************
//
//! The number of individual CGI URIs that are configured for this system.
//
//*****************************************************************************
#define NUM_CONFIG_CGI_URIS     (sizeof(g_psConfigCGIURIs) / sizeof(tCGI))

//*****************************************************************************
//
//! The file sent back to the browser by default following completion of any
//! of our CGI handlers.  Each individual handler returns the URI of the page
//! to load in response to it being called.
//
//*****************************************************************************
#define DEFAULT_CGI_RESPONSE    "/io_cgi.shtml"

//*****************************************************************************
//
//! The file sent back to the browser in cases where a parameter error is
//! detected by one of the CGI handlers.  This should only happen if someone
//! tries to access the CGI directly via the broswer command line and doesn't
//! enter all the required parameters alongside the URI.
//
//*****************************************************************************
#define PARAM_ERROR_RESPONSE    "/perror.html"

#define JAVASCRIPT_HEADER                                                     \
    "<script type='text/javascript' language='JavaScript'><!--\n"
#define JAVASCRIPT_FOOTER                                                     \
    "//--></script>\n"

//*****************************************************************************
//
// Timeout for DHCP address request (in seconds).
//
//*****************************************************************************
#ifndef DHCP_EXPIRE_TIMER_SECS
#define DHCP_EXPIRE_TIMER_SECS  45
#endif

//*****************************************************************************
//
// This CGI handler is called whenever the web browser requests iocontrol.cgi.
//
//*****************************************************************************
static char *
ControlCGIHandler(int iIndex, int iNumParams, char *pcParam[], char *pcValue[])
{
    long  lPC7State, lPC6State, lPC5State, lPC4State,
          lPE3State, lPE2State, lPE1State, lPE0State;

    //
    // Get each of the expected parameters.
    //
    lPC7State = FindCGIParameter("PC7On", pcParam, iNumParams);
    lPC6State = FindCGIParameter("PC6On", pcParam, iNumParams);
    lPC5State = FindCGIParameter("PC5On", pcParam, iNumParams);
    lPC4State = FindCGIParameter("PC4On", pcParam, iNumParams);
    lPE3State = FindCGIParameter("PE3On", pcParam, iNumParams);
    lPE2State = FindCGIParameter("PE2On", pcParam, iNumParams);
    lPE1State = FindCGIParameter("PE1On", pcParam, iNumParams);
    lPE0State = FindCGIParameter("PE0On", pcParam, iNumParams);
    
    //
    // We got all the parameters and the values were within the expected ranges
    // so go ahead and make the changes.
    //
    io_set_port(GPIO_PORTC_BASE, GPIO_PIN_7, (lPC7State == -1) ? false : true);
    io_set_port(GPIO_PORTC_BASE, GPIO_PIN_6, (lPC6State == -1) ? false : true);
    io_set_port(GPIO_PORTC_BASE, GPIO_PIN_5, (lPC5State == -1) ? false : true);
    io_set_port(GPIO_PORTC_BASE, GPIO_PIN_4, (lPC4State == -1) ? false : true);
    io_set_port(GPIO_PORTE_BASE, GPIO_PIN_3, (lPE3State == -1) ? false : true);
    io_set_port(GPIO_PORTE_BASE, GPIO_PIN_2, (lPE2State == -1) ? false : true);
    io_set_port(GPIO_PORTE_BASE, GPIO_PIN_1, (lPE1State == -1) ? false : true);
    io_set_port(GPIO_PORTE_BASE, GPIO_PIN_0, (lPE0State == -1) ? false : true);

    //
    // Send back the default response page.
    //
    return(DEFAULT_CGI_RESPONSE);
}

//*****************************************************************************
//
// This CGI handler is called whenever the web browser requests adcinput.cgi.
//
//*****************************************************************************
static char *
AdcCGIHandler(int iIndex, int iNumParams, char *pcParam[], char *pcValue[])
{
    long lADCParam;
    
    //
    // Find the parameter that has the value we need to display.
    //
    lADCParam = FindCGIParameter("UpdateADC", pcParam, iNumParams);
    
    //
    // If the parameter was not found, show the error page.
    //
    if(lADCParam == -1)
    {
        return(PARAM_ERROR_RESPONSE);
    }
    
    return(DEFAULT_CGI_RESPONSE);
}

//*****************************************************************************
//
// This CGI handler is called whenever the web browser requests settxt.cgi.
//
//*****************************************************************************
static char *
SetTextCGIHandler(int iIndex, int iNumParams, char *pcParam[], char *pcValue[])
{
    long lStringParam;
    char pcDecodedString[24];

    //
    // Find the parameter that has the string we need to display.
    //
    lStringParam = FindCGIParameter("DispText", pcParam, iNumParams);

    //
    // If the parameter was not found, show the error page.
    //
    if(lStringParam == -1)
    {
        return(PARAM_ERROR_RESPONSE);
    }

    //
    // The parameter is present. We need to decode the text for display.
    //
    DecodeFormString(pcValue[lStringParam], pcDecodedString, 24);

    //
    // Print the new decoded string.
    //
    UARTprintf("Browser Message: %s\n", pcDecodedString);

    //
    // Tell the HTTPD server which file to send back to the client.
    //
    return(DEFAULT_CGI_RESPONSE);
}

//*****************************************************************************
//
// This function is called by the HTTP server whenever it encounters an SSI
// tag in a web page.  The iIndex parameter provides the index of the tag in
// the g_pcConfigSSITags array. This function writes the substitution text
// into the pcInsert array, writing no more than iInsertLen characters.
//
//*****************************************************************************
static int
SSIHandler(int iIndex, char *pcInsert, int iInsertLen)
{
    int port, pin;
    //
    // Which SSI tag have we been passed?
    //
    if(iIndex >=0 && iIndex <= 7)
    {
        //
        // Get GPIO state.
        //
        if(iIndex < 4)
            port = GPIO_PORTE_BASE;
        else
            port = GPIO_PORTC_BASE;
        
        pin = 1 << iIndex;
        
        //
        // Get port state.
        //
        io_get_portstate(port, pin, pcInsert, iInsertLen);
    }
    else if(iIndex >= 8 && iIndex <= 15)
    {
      //
      // Get pin
      //
      pin = iIndex - 8;
      
      //
      // Get ADC value.
      //
      io_get_adcvalue(pin, pcInsert, iInsertLen);
    }
    else if(iIndex == 16)
    {
      //
      // Get FORMVARS.
      //
      usnprintf(pcInsert, iInsertLen,
                      "%spc7s=%d;\npc6s=%d;\npc5s=%d;\npc4s=%d;\npe3s=%d;\npe2s=%d;\npe1s=%d;\npe0s=%d;\n%s",
                      JAVASCRIPT_HEADER,
                      io_is_port_on(GPIO_PORTC_BASE, GPIO_PIN_7),
                      io_is_port_on(GPIO_PORTC_BASE, GPIO_PIN_6),
                      io_is_port_on(GPIO_PORTC_BASE, GPIO_PIN_5),
                      io_is_port_on(GPIO_PORTC_BASE, GPIO_PIN_4),
                      io_is_port_on(GPIO_PORTE_BASE, GPIO_PIN_3),
                      io_is_port_on(GPIO_PORTE_BASE, GPIO_PIN_2),
                      io_is_port_on(GPIO_PORTE_BASE, GPIO_PIN_1),
                      io_is_port_on(GPIO_PORTE_BASE, GPIO_PIN_0),
                      JAVASCRIPT_FOOTER);
    }

    //
    // Tell the server how many characters our insert string contains.
    //
    return(strlen(pcInsert));
}

//*****************************************************************************
//
// Display an lwIP type IP Address.
//
//*****************************************************************************
void
DisplayIPAddress(unsigned long ipaddr)
{
    char pucBuf[16];
    unsigned char *pucTemp = (unsigned char *)&ipaddr;

    //
    // Convert the IP Address into a string.
    //
    usprintf(pucBuf, "%d.%d.%d.%d", pucTemp[0], pucTemp[1], pucTemp[2],
             pucTemp[3]);

    //
    // Display the string.
    //
    UARTprintf("%s\n", pucBuf);
}

//*****************************************************************************
//
// The interrupt handler for the SysTick interrupt.
//
//*****************************************************************************
void
SysTickIntHandler(void)
{
    //
    // Indicate that a SysTick interrupt has occurred.
    //
    HWREGBITW(&g_ulFlags, FLAG_SYSTICK) = 1;

    //
    // Call the lwIP timer handler.
    //
    lwIPTimer(SYSTICKMS);
}

//*****************************************************************************
//
// Required by lwIP library to support any host-related timer functions.
//
//*****************************************************************************
void
lwIPHostTimerHandler(void)
{
    static unsigned long ulLastIPAddress = 0;
    unsigned long ulIPAddress;

    ulIPAddress = lwIPLocalIPAddrGet();

    //
    // If IP Address has not yet been assigned, update the display accordingly
    //
    if(ulIPAddress == 0)
    {
        static int iColumn = 0;

        //
        // Update status bar on the display.
        //
        if(iColumn < 1)
        {
            UARTprintf("< *");
        }
        else
        {
            UARTprintf("*");
        }

        iColumn++;
        if(iColumn > 74)
        {
            iColumn = 0;
            UARTprintf(" >\n");
        }
    }

    //
    // Check if IP address has changed, and display if it has.
    //
    else if(ulLastIPAddress != ulIPAddress)
    {
        ulLastIPAddress = ulIPAddress;
        UARTprintf("\nIP:   ");
        DisplayIPAddress(ulIPAddress);
        ulIPAddress = lwIPLocalNetMaskGet();
        UARTprintf("MASK: ");
        DisplayIPAddress(ulIPAddress);
        ulIPAddress = lwIPLocalGWAddrGet();
        UARTprintf("GW:   ");
        DisplayIPAddress(ulIPAddress);
    }
}

//*****************************************************************************
//
// This example demonstrates the use of the Ethernet Controller and lwIP
// TCP/IP stack to control various peripherals on the board via a web
// browser.
//
//*****************************************************************************
int
main(void)
{
    unsigned long ulUser0, ulUser1;
    unsigned char pucMACArray[8];

    //
    // If running on Rev A2 silicon, turn the LDO voltage up to 2.75V.  This is
    // a workaround to allow the PLL to operate reliably.
    //
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

    //
    // Set the processor to run at 50 MHz
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);
    //HWREG(NVIC_VTABLE) = (unsigned long)g_pfnVectors;

    //
    // Enable the peripherals and set GPIO pins for UART.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);
    UARTprintf("Web-Based I/O Control\n");

    //
    // Enable and Reset the Ethernet Controller.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ETH);
    SysCtlPeripheralReset(SYSCTL_PERIPH_ETH);
    IntRegister(INT_ETH, lwIPEthernetIntHandler);

    //
    // Enable Port F for Ethernet LEDs.
    //  LED0        Bit 3   Output
    //  LED1        Bit 2   Output
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    GPIODirModeSet(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3, GPIO_DIR_MODE_HW);
    GPIOPadConfigSet(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3,
                     GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD);

    //
    // Configure SysTick for a periodic interrupt.
    //
    SysTickPeriodSet(SysCtlClockGet() / SYSTICKHZ);
    SysTickEnable();
    IntRegister(FAULT_SYSTICK, SysTickIntHandler);
    SysTickIntEnable();

    //
    // Enable processor interrupts.
    //
    IntMasterEnable();

    //
    // Configure the hardware MAC address for Ethernet Controller filtering of
    // incoming packets.
    //
    // For the Eagle 100 SBC, the MAC address will be stored in the
    // non-volatile USER0 and USER1 registers.  These registers can be read
    // using the FlashUserGet function, as illustrated below.
    //
    FlashUserGet(&ulUser0, &ulUser1);
    if((ulUser0 == 0xffffffff) || (ulUser1 == 0xffffffff))
    {
        //
        // We should never get here.  This is an error if the MAC address has
        // not been programmed into the device.  Exit the program.
        //
        UARTprintf("MAC Address\nNot Programmed!\n");
        while(1);
    }

    //
    // Convert the 24/24 split MAC address from NV ram into a 32/16 split
    // MAC address needed to program the hardware registers, then program
    // the MAC address into the Ethernet Controller registers.
    //
    pucMACArray[0] = ((ulUser0 >>  0) & 0xff);
    pucMACArray[1] = ((ulUser0 >>  8) & 0xff);
    pucMACArray[2] = ((ulUser0 >> 16) & 0xff);
    pucMACArray[3] = ((ulUser1 >>  0) & 0xff);
    pucMACArray[4] = ((ulUser1 >>  8) & 0xff);
    pucMACArray[5] = ((ulUser1 >> 16) & 0xff);

    //
    // Initialze the lwIP library, using DHCP.
    //
#ifdef USE_STATIC_IP
    lwIPInit(pucMACArray,
             IP_ADDR(192, 168, 1, 200), // StaticIP
             IP_ADDR(255, 255, 255, 0), // SubnetMask
             IP_ADDR(192, 168, 1, 253), // GatewayIP
             IPADDR_USE_STATIC);
#else
    lwIPInit(pucMACArray, 0, 0, 0, IPADDR_USE_DHCP);
#endif

    //
    // Initialize a sample httpd server.
    //
    httpd_init();

    //
    // Pass our tag information to the HTTP server.
    //
    http_set_ssi_handler(SSIHandler, g_pcConfigSSITags,
                         NUM_CONFIG_SSI_TAGS);

    //
    // Pass our CGI handlers to the HTTP server.
    //
    http_set_cgi_handlers(g_psConfigCGIURIs, NUM_CONFIG_CGI_URIS);

    //
    // Initialize IO controls
    //
    io_init();

    //
    // Loop forever.  All the work is done in interrupt handlers.
    //
    while(1)
    {
      
    }
}
