//*****************************************************************************
//
// io.c - I/O routines for the enet_io example application.
//
// Copyright (c) 2007-2009 Texas Instruments Incorporated. All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of the Micromint Eagle Firmware Package.
//
//*****************************************************************************

#include "inc/hw_adc.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/adc.h"
#include "driverlib/sysctl.h"
#include "utils/ustdlib.h"
#include "io.h"

//*****************************************************************************
//
// Initialize the IO used in this demo
// 1. STATUS LED on Port F pin 0
// 2. Upper nibble on Port C (PC4-7)
//
//*****************************************************************************
void
io_init(void)
{
    int i;
    
    //
    // Enable PC4-7 for output.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    GPIOPinTypeGPIOOutput(GPIO_PORTC_BASE, 0xf0);
    GPIOPinWrite(GPIO_PORTC_BASE, 0xf0, 0);
    
    //
    // Enable PE0-3 for output.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    GPIOPinTypeGPIOOutput(GPIO_PORTE_BASE, 0xf);
    GPIOPinWrite(GPIO_PORTE_BASE, 0xf, 0);
    
    //
    // Enable the ADC.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC);
    SysCtlADCSpeedSet(SYSCTL_ADCSPEED_500KSPS);
    
    //
    // Enable sequencer 0 for all 8 channels of ADC.
    //
    ADCSequenceConfigure(ADC_BASE, 0, ADC_TRIGGER_PROCESSOR, 0);
    
    //
    // Enable all ADC channels.
    //
    for (i = 0; i < (NUM_ADCS - 1); i++)
    {
        ADCSequenceStepConfigure(ADC_BASE, 0, i, ADC_CTL_CH(i));
    }
    ADCSequenceStepConfigure(ADC_BASE, 0, (NUM_ADCS - 1),
                             ADC_CTL_IE | ADC_CTL_END | ADC_CTL_CH(NUM_ADCS - 1));
    
    //
    // Enable the sequence.
    //
    ADCSequenceEnable(ADC_BASE, 0);   
}

//*****************************************************************************
//
// Get ADC value
//
//*****************************************************************************
void
io_get_adcvalue(unsigned long pin, char *pcBuf, int iBufLen)
{
    unsigned long adc_sample[8];
    char cInt[5];
    int i;
  
    //
    // Trigger the sequence 0.
    //
    ADCProcessorTrigger(ADC_BASE, 0);
    
    //
    // Wait until the sample sequence has completed.
    //
    while(!ADCIntStatus(ADC_BASE, 0, false))
    {
    }

    //
    // Clear the interrupt flag.
    //
    HWREG(ADC_BASE + ADC_O_ISC) |= ADC_ISC_IN0;
    ADCSequenceDataGet(ADC_BASE, 0, adc_sample);
    
    //
    // Convert integer to string.
    //
    cInt[0] = (adc_sample[pin] / 1000) + 48;
    cInt[1] = ((adc_sample[pin] / 100) % 10) + 48;
    cInt[2] = ((adc_sample[pin] / 10) % 10) + 48;
    cInt[3] = (adc_sample[pin] % 10) + 48;
    cInt[4] = '\0';
    
    //
    // Do not display leading zero's.
    //
    for(i = 0; i < 3; i++)
    {
      if(cInt[i] == '0')
        cInt[i] = ' ';
    }
  
    //
    // Give the string to the browser.
    //
    usnprintf(pcBuf, iBufLen, (char const *)cInt);
}

//*****************************************************************************
//
// Set port data
//
//*****************************************************************************
void
io_set_port(unsigned long port, unsigned long pin, tBoolean bOn)
{
    //
    // Set pin data
    //
    GPIOPinWrite(port, pin, bOn ? pin : 0);
}

//*****************************************************************************
//
// Return port state
//
//*****************************************************************************
void
io_get_portstate(unsigned long port, unsigned long pin, char * pcBuf, int iBufLen)
{
    //
    // Get the state of the pin
    //
    usnprintf(pcBuf, iBufLen,
              GPIOPinRead(port, pin) ? "ON" : "OFF");
}

//*****************************************************************************
//
// Return port state as an integer, 1 on, 0 off.
//
//*****************************************************************************
int
io_is_port_on(unsigned long port, unsigned long pin)
{
    //
    // Get the state of the pin
    //
    return (GPIOPinRead(port, pin) ? 1 : 0);
}
