/*
  enet_snmp.c - SNMP and WebServer Application using lwIP
                by Micromint Support <support@micromint.com>    Feb-2009

The lwIP private MIB (enterprises.26381) currently has 10 sensor IDs defined
but can be easily changed to support many more. To execute them you can use
the command line net-snmp tools for Windows from

http://www.net-snmp.org/download.html

The demo currently uses the first 4 sensor IDs:

enterprises.26381.1.1.1.1.
  0 - LED at PE1 (read/write)
  1 - User button at PA6 (read only)
  2 - Nibble PB0-3 configured as input (read only)
  3 - Nibble PC4-7 configured as output (read/write)

Representative commands (board at 192.168.1.220):

snmpwalk -v 1 -c public 192.168.1.220
  List all base MIBs
snmpwalk -v 1 -c public 192.168.1.220 enterprises
  List all private MIBs
snmpget -v 1 -c public 192.168.1.220 sysContact.0
  Get contact information
snmpset -v 1 -c public 192.168.1.220 sysContact.0 s "support@micromint.com"
  Set contact information
snmpget -v 1 -c public 192.168.1.220 enterprises.26381.1.1.1.1.0
  Get current LED state
snmpset -v 1 -c public 192.168.1.220 enterprises.26381.1.1.1.1.0 i 0
  Turn LED off
snmpset -v 1 -c public 192.168.1.220 enterprises.26381.1.1.1.1.0 i 1
  Turn LED on
snmpset -v 1 -c public 192.168.1.220 enterprises.26381.1.1.1.1.3 i 10
  Set nibble PC4-7 to '1010'

  Copyright (c) 2008-2010 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_gpio.h"
#include "inc/hw_nvic.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/ethernet.h"
#include "driverlib/flash.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "utils/lwiplib.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "httpserver_raw/httpd.h"

#define LED_PIN                 GPIO_PIN_1
#define LED_GPIO_BASE           GPIO_PORTE_BASE
#define LED_REG                 HWREG(LED_GPIO_BASE + (GPIO_O_DATA + (LED_PIN << 2)))

#ifdef USE_STATIC_IP
/* Convert octets to integer */
#define IPADDR(addr, a, b, c, d) \
        *addr = (((u32_t)((a) & 0xff) << 24) | \
                 ((u32_t)((b) & 0xff) << 16) | \
                 ((u32_t)((c) & 0xff) << 8) | \
                  (u32_t)((d) & 0xff))

unsigned long ulStaticIP;
unsigned long ulSubnetMask;
unsigned long ulGatewayIP;
#endif

#if LWIP_SNMP
#include "lwip/snmp.h"
#include "lwip/snmp_msg.h"
#include "private_mib.h"

u8_t SYSCONTACT[] = "support@micromint.com";
u8_t SIZE_SYSCONTACT = sizeof(SYSCONTACT) - 1;
u8_t SYSLOC[] = "Lake Mary, FL";
u8_t SIZE_SYSLOC = sizeof(SYSLOC) - 1;
u8_t SYSNAME[] = "Micromint Eagle 100";
u8_t SIZE_SYSNAME = sizeof(SYSNAME) - 1;
u8_t SYSDESCR[] = "Industrial Controller SBC";
u8_t SIZE_SYSDESCR = sizeof(SYSDESCR) - 1;
u8_t AUTHENTRAPS[] = "1234";

static unsigned long LedOverride = 0;
static struct ip_addr IpSnmpDest;

unsigned long SensorRead(unsigned long);
unsigned long SensorCheck(unsigned long, void*);
unsigned long SensorWrite(unsigned long, void*);
#endif

/*
  ! \addtogroup example_list
  ! <h1>SNMP with lwIP (enet_snmp)</h1>
  !
  ! This example application demonstrates the operation of the Stellaris
  ! Ethernet controller using the lwIP TCP/IP Stack.  DHCP is used to obtain
  ! an Ethernet address.  If DHCP times out without obtaining an address,
  ! AUTOIP will be used to obtain a link-local address.  The address that is
  ! selected will be shown on the serial port.
  ! 
  ! The file system code will first check to see if an SD card has been plugged
  ! into the microSD slot.  If so, all file requests from the web server will
  ! be directed to the SD card.  Otherwise, a default set of pages served up
  ! by an internal file system will be used.
  ! 
  ! For additional details on lwIP, refer to the lwIP web page at:
  ! http://www.sics.se/~adam/lwip/
*/

/*
   Defines for setting up the system clock.
*/
#define SYSTICKHZ               100
#define SYSTICKMS               (1000 / SYSTICKHZ)
#define SYSTICKUS               (1000000 / SYSTICKHZ)
#define SYSTICKNS               (1000000000 / SYSTICKHZ)

/*
   Interrupt priority definitions.  The top 3 bits of these values are
   significant with lower values indicating higher priority interrupts.
*/
#define SYSTICK_INT_PRIORITY    0x80
#define ETHERNET_INT_PRIORITY   0xC0

/*
  External Application references.
*/
extern void fs_init(void);
extern void fs_tick(unsigned long ulTickMS);

/*
  Display an lwIP type IP Address.
*/
void DisplayIPAddress(unsigned long ipaddr)
{
    char pucBuf[16];
    unsigned char *pucTemp = (unsigned char *)&ipaddr;

    /* Convert the IP Address into a string. */
    usprintf(pucBuf, "%d.%d.%d.%d", pucTemp[0], pucTemp[1], pucTemp[2],
             pucTemp[3]);

    /* Display the string. */
    UARTprintf("%s\n", pucBuf);
}

/*
  Required by lwIP library to support any host-related timer functions.
*/
void lwIPHostTimerHandler(void)
{
    static unsigned long ulLastIPAddress = 0;
    unsigned long ulIPAddress;

    ulIPAddress = lwIPLocalIPAddrGet();

    /* If IP Address has not yet been assigned, update the display */
    if(ulIPAddress == 0)
    {
        static int iColumn = 0;

        /* Update status bar on the display */
        if(iColumn < 1)
        {
            UARTprintf("< *");
        }
        else
        {
            UARTprintf("*");
        }

        iColumn++;
        if(iColumn > 74)
        {
            iColumn = 0;
            UARTprintf(" >\n");
        }
    }

    /* Check if IP address has changed, and display if it has */
    else if(ulLastIPAddress != ulIPAddress)
    {
        ulLastIPAddress = ulIPAddress;
        UARTprintf("\nIP:   ");
        DisplayIPAddress(ulIPAddress);
        ulIPAddress = lwIPLocalNetMaskGet();
        UARTprintf("MASK: ");
        DisplayIPAddress(ulIPAddress);
        ulIPAddress = lwIPLocalGWAddrGet();
        UARTprintf("GW:   ");
        DisplayIPAddress(ulIPAddress);
    }
}

/*
  Handler for SysTick interrupt
*/
void SysTickIntHandler(void)
{
    static unsigned long tickCount;

    /* Call the lwIP timer handler. */
    lwIPTimer(SYSTICKMS);

#if LWIP_SNMP
    /* Update SNMP uptime timestamp */
    snmp_inc_sysuptime();
#endif // LWIP_SNMP

    /* Run the file system tick handler */
    fs_tick(SYSTICKMS);

    /* Periodically Toggle LED */
    tickCount++;
    if (tickCount > SYSTICKHZ)
    {
        if (LedOverride == 0)
            LED_REG ^= LED_PIN;
        tickCount = 0;
    }
}

#if LWIP_SNMP
/*
  Called from snmpget to read sensor
*/
unsigned long SensorRead(unsigned long id)
{
    unsigned long retValue = 0;

    switch (id)
    {
      case 0x00: /* LED at PE1 */
          retValue = (LED_REG == LED_PIN);
          break;
      case 0x01: /* User button at PA6 */
          /* Returns 0 when pressed */
          retValue = (GPIOPinRead(GPIO_PORTA_BASE, GPIO_PIN_6) == GPIO_PIN_6);
          break;
      case 0x02: /* Port PB0-3 (4 bits) */
          retValue = GPIOPinRead(GPIO_PORTB_BASE, 0x0f);
          break;
      case 0x03: /* Port PC4-7 (4 bits) */
          retValue = (GPIOPinRead(GPIO_PORTC_BASE, 0xf0) >> 4);
          break;
      default:
          retValue = 0xff;
          break;
    }

    //UARTprintf("SensorRead: id=%d data=%d\n", id, retValue);
    return retValue;
}

/*
  Check if sensor supports value and
    return 0 if change is not feasible
*/
unsigned long SensorCheck(unsigned long id, void *data)
{
    unsigned long retValue = 0;

    switch (id)
    {
      case 0x00: /* LED at PE1 */
      case 0x03: /* Port PC4-7 (4 bits) */
          retValue = 1;
          break;
      /* Can't write on push button or other input ports */
      case 0x01: /* User button at PA6 */
      case 0x02: /* Port PB0-3 (4 bits) */
      default:
          break;
    }

    return retValue;
}

/*
  Called from snmpset to set sensor
*/
unsigned long SensorWrite(unsigned long id, void *data)
{
    unsigned long retValue = 0;
    unsigned long *intPtr = data;
    unsigned long intData = (*intPtr) & 0xffff; /* 16-bit integers */

    switch (id)
    {
      case 0x00: /* LED at PE1 */
          LED_REG = (intData != 0) ? LED_PIN : 0;
          LedOverride = 1; /* Stop blinking */
          retValue = 1;
          break;
      case 0x03: /* Port PC4-7 (4 bits) */
          GPIOPinWrite(GPIO_PORTC_BASE, 0xf0, (intData << 4));
          retValue = 1;
      /* Can't write on push button or other input ports */
      case 0x01: /* User button at PA6 */
      case 0x02: /* Port PB0-3 (4 bits) */
      default:
          break;
    }

    //UARTprintf("SensorWrite: id=%d data=%d\n", id, intData);
    return retValue;
}
#endif /* LWIP_SNMP */

/*
  Main function to use SNMP with lwIP
*/
int main(void)
{
    unsigned long ulUser0, ulUser1;
    unsigned char pucMACArray[8];

    /* If running on Rev A2 silicon, turn the LDO voltage up to 2.75V */
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

    /* Set the processor to run at 50 MHz */
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);

    /* Enable LED */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    HWREG(LED_GPIO_BASE + GPIO_O_DIR) |= LED_PIN;
    HWREG(LED_GPIO_BASE + GPIO_O_DEN) |= LED_PIN;

    /* Enable the peripherals and set GPIO pins for UART. */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);
    UARTprintf("SNMP with lwIP\n");

    /* Enable GPIOs: Button PA6 for input */
    GPIOPinTypeGPIOInput(GPIO_PORTA_BASE, GPIO_PIN_6);
    GPIOPadConfigSet(GPIO_PORTA_BASE, GPIO_PIN_6, GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD_WPU);
    /* PB0-3 for input */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    GPIOPinTypeGPIOInput(GPIO_PORTB_BASE, 0x0f);
    GPIOPadConfigSet(GPIO_PORTB_BASE, 0x0f, GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD_WPU);
    /* PC4-7 for output */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    GPIOPinTypeGPIOOutput(GPIO_PORTC_BASE, 0xf0);  
   
    /* Enable and Reset the Ethernet Controller. */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ETH);
    SysCtlPeripheralReset(SYSCTL_PERIPH_ETH);
    IntRegister(INT_ETH, lwIPEthernetIntHandler);

    /*
      Enable Port F for Ethernet LEDs.
      LED0        Bit 3   Output
      LED1        Bit 2   Output
    */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    GPIODirModeSet(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3, GPIO_DIR_MODE_HW);
    GPIOPadConfigSet(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3,
                     GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD);

    /* Configure SysTick for a periodic interrupt */
    SysTickPeriodSet(SysCtlClockGet() / SYSTICKHZ);
    SysTickEnable();
    IntRegister(FAULT_SYSTICK, SysTickIntHandler);
    SysTickIntEnable();

    /* Enable processor interrupts */
    IntMasterEnable();

    /* Initialize the file system */
    fs_init();

    /*
      Configure the hardware MAC address for Ethernet Controller filtering of
      incoming packets.

      For the Eagle SBC, the MAC address will be stored in the
      non-volatile USER0 and USER1 registers.  These registers can be read
      using the FlashUserGet function, as illustrated below.
    */
    FlashUserGet(&ulUser0, &ulUser1);
    if((ulUser0 == 0xffffffff) || (ulUser1 == 0xffffffff))
    {
        /* Report error and exit if MAC address is not programmed */
        UARTprintf("MAC Address\nNot Programmed!\n");
        while(1) {}
    }

    /*
      Convert the 24/24 split MAC address from NV ram into a 32/16 split MAC
      address needed to program the hardware registers, then program the MAC
      address into the Ethernet Controller registers.
    */
    pucMACArray[0] = ((ulUser0 >>  0) & 0xff);
    pucMACArray[1] = ((ulUser0 >>  8) & 0xff);
    pucMACArray[2] = ((ulUser0 >> 16) & 0xff);
    pucMACArray[3] = ((ulUser1 >>  0) & 0xff);
    pucMACArray[4] = ((ulUser1 >>  8) & 0xff);
    pucMACArray[5] = ((ulUser1 >> 16) & 0xff);

    /* Initialze the lwIP TCP/IP stack */
#ifdef USE_STATIC_IP
    IPADDR(&ulStaticIP,  192, 168, 1, 200);
    IPADDR(&ulSubnetMask, 255, 255, 255, 0);
    IPADDR(&ulGatewayIP, 192, 168, 1, 1);
    lwIPInit(pucMACArray, ulStaticIP, ulSubnetMask, ulGatewayIP, IPADDR_USE_STATIC);
#else
    /* Use DHCP */
    lwIPInit(pucMACArray, 0, 0, 0, IPADDR_USE_DHCP);
    /* Indicate that DHCP has started */
    UARTprintf("Waiting for IP\n");
#endif

#if LWIP_SNMP
#if SNMP_PRIVATE_MIB
    /* Initialize private MIB */
    lwip_privmib_init();
#endif

    /* Setup SNMP system parameters */
    snmp_set_syscontact(SYSCONTACT, &SIZE_SYSCONTACT);
    snmp_set_syslocation(SYSLOC, &SIZE_SYSLOC);
    snmp_set_sysdesr(SYSDESCR, &SIZE_SYSDESCR);
    snmp_set_sysname(SYSNAME, &SIZE_SYSNAME);
    snmp_set_snmpenableauthentraps(AUTHENTRAPS);

    /* Set destination for SNMP traps */
    IP4_ADDR(&IpSnmpDest, 192, 168, 0, 1);
    snmp_trap_dst_enable(0, 1);
    snmp_trap_dst_ip_set(0, &IpSnmpDest);

    /* Initialize SNMP client agent */
    snmp_init();
#endif

    /* Initialize httpd server */
    httpd_init();

    /*
       Set the interrupt priorities.  We set the SysTick interrupt to a higher
       priority than the Ethernet interrupt to ensure that the file system
       tick is processed if SysTick occurs while the Ethernet handler is being
       processed.  This is very likely since all the TCP/IP and HTTP work is
       done in the context of the Ethernet interrupt.
    */
    IntPriorityGroupingSet(4);
    IntPrioritySet(INT_ETH, ETHERNET_INT_PRIORITY);
    IntPrioritySet(FAULT_SYSTICK, SYSTICK_INT_PRIORITY);

    /* Loop while waiting for interrupts */
    while(1) {}
}
