/*
  enet_tftpd.c - Sample TFTP server using lwIP.
                 by Micromint Support <support@micromint.com>    Aug-2009

  Copyright (c) 2008-2010 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/ethernet.h"
#include "driverlib/flash.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "utils/lwiplib.h"
#include "utils/tftp.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "tftp_fs.h"

#ifdef USE_STATIC_IP
/* Convert octets to integer */
#define IPADDR(addr, a, b, c, d) \
        *addr = (((u32_t)((a) & 0xff) << 24) | \
                 ((u32_t)((b) & 0xff) << 16) | \
                 ((u32_t)((c) & 0xff) << 8) | \
                  (u32_t)((d) & 0xff))

unsigned long ulStaticIP;
unsigned long ulSubnetMask;
unsigned long ulGatewayIP;
#endif

/*
  ! \addtogroup example_list
  ! <h1>TFTP with lwIP (enet_tftp)</h1>
  !
  ! This example application demonstrates the operation of the Stellaris
  ! Ethernet controller using the lwIP TCP/IP Stack.  DHCP is used to obtain
  ! an Ethernet address.  If DHCP times out without obtaining an address,
  ! AUTOIP will be used to obtain a link-local address.  The address that is
  ! selected will be shown on the serial port.
  ! 
  ! The file system code will first check to see if an SD card has been plugged
  ! into the microSD slot.  If so, all file requests from the TFTP server will
  ! be directed to the SD card.  Otherwise, a default set of pages served up
  ! by an internal file system will be used.
  ! 
  ! For additional details on lwIP, refer to the lwIP web page at:
  ! http://www.sics.se/~adam/lwip/
*/

/*
   Defines for setting up the system clock.
*/
#define SYSTICKHZ               100
#define SYSTICKMS               (1000 / SYSTICKHZ)
#define SYSTICKUS               (1000000 / SYSTICKHZ)
#define SYSTICKNS               (1000000000 / SYSTICKHZ)

/*
   Interrupt priority definitions.  The top 3 bits of these values are
   significant with lower values indicating higher priority interrupts.
*/
#define SYSTICK_INT_PRIORITY    0x80
#define ETHERNET_INT_PRIORITY   0xC0

/*
  External Application references.
*/
extern void fs_init(void);
extern void fs_tick(unsigned long ulTickMS);

/*
  Display an lwIP type IP Address.
*/
void DisplayIPAddress(unsigned long ipaddr)
{
    char pucBuf[16];
    unsigned char *pucTemp = (unsigned char *)&ipaddr;

    /* Convert the IP Address into a string. */
    usprintf(pucBuf, "%d.%d.%d.%d", pucTemp[0], pucTemp[1], pucTemp[2],
             pucTemp[3]);

    /* Display the string. */
    UARTprintf("%s\n", pucBuf);
}

/*
  Required by lwIP library to support any host-related timer functions.
*/
void lwIPHostTimerHandler(void)
{
    static unsigned long ulLastIPAddress = 0;
    unsigned long ulIPAddress;

    ulIPAddress = lwIPLocalIPAddrGet();

    /* If IP Address has not yet been assigned, update the display */
    if(ulIPAddress == 0)
    {
        static int iColumn = 0;

        /* Update status bar on the display */
        if(iColumn < 1)
        {
            UARTprintf("< *");
        }
        else
        {
            UARTprintf("*");
        }

        iColumn++;
        if(iColumn > 74)
        {
            iColumn = 0;
            UARTprintf(" >\n");
        }
    }

    /* Check if IP address has changed, and display if it has */
    else if(ulLastIPAddress != ulIPAddress)
    {
        ulLastIPAddress = ulIPAddress;
        UARTprintf("\nIP:   ");
        DisplayIPAddress(ulIPAddress);
        ulIPAddress = lwIPLocalNetMaskGet();
        UARTprintf("MASK: ");
        DisplayIPAddress(ulIPAddress);
        ulIPAddress = lwIPLocalGWAddrGet();
        UARTprintf("GW:   ");
        DisplayIPAddress(ulIPAddress);
    }
}

/*
  Handler for SysTick interrupt
*/
void SysTickIntHandler(void)
{
    /* Call the lwIP timer handler. */
    lwIPTimer(SYSTICKMS);

#ifdef CONFIG_MMC
    /* Run the file system tick handler */
    fs_tick(SYSTICKMS);
#endif // CONFIG_MMC
}

/*
  Main function to use TFTP with lwIP
*/
int main(void)
{
    unsigned long ulUser0, ulUser1;
    unsigned char pucMACArray[8];

    /* If running on Rev A2 silicon, turn the LDO voltage up to 2.75V */
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

    /* Set the processor to run at 50 MHz */
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);

    /* Enable the peripherals and set GPIO pins for UART. */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);
    UARTprintf("Ethernet with lwIP\n");

    /* Enable and Reset the Ethernet Controller. */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ETH);
    SysCtlPeripheralReset(SYSCTL_PERIPH_ETH);
    IntRegister(INT_ETH, lwIPEthernetIntHandler);

    /*
      Enable Port F for Ethernet LEDs.
      LED0        Bit 3   Output
      LED1        Bit 2   Output
    */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    GPIODirModeSet(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3, GPIO_DIR_MODE_HW);
    GPIOPadConfigSet(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3,
                     GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD);

    /* Configure SysTick for a periodic interrupt */
    SysTickPeriodSet(SysCtlClockGet() / SYSTICKHZ);
    SysTickEnable();
    IntRegister(FAULT_SYSTICK, SysTickIntHandler);
    SysTickIntEnable();

    /* Enable processor interrupts */
    IntMasterEnable();

    /* Initialize the file system */
    fs_init();

    /*
      Configure the hardware MAC address for Ethernet Controller filtering of
      incoming packets.

      For the Eagle SBC, the MAC address will be stored in the
      non-volatile USER0 and USER1 registers.  These registers can be read
      using the FlashUserGet function, as illustrated below.
    */
    FlashUserGet(&ulUser0, &ulUser1);
    if((ulUser0 == 0xffffffff) || (ulUser1 == 0xffffffff))
    {
        /* Report error and exit if MAC address is not programmed */
        UARTprintf("MAC Address\nNot Programmed!\n");
        while(1)
        {
        }
    }

    /*
      Convert the 24/24 split MAC address from NV ram into a 32/16 split MAC
      address needed to program the hardware registers, then program the MAC
      address into the Ethernet Controller registers.
    */
    pucMACArray[0] = ((ulUser0 >>  0) & 0xff);
    pucMACArray[1] = ((ulUser0 >>  8) & 0xff);
    pucMACArray[2] = ((ulUser0 >> 16) & 0xff);
    pucMACArray[3] = ((ulUser1 >>  0) & 0xff);
    pucMACArray[4] = ((ulUser1 >>  8) & 0xff);
    pucMACArray[5] = ((ulUser1 >> 16) & 0xff);

    /* Initialze the lwIP TCP/IP stack */
#ifdef USE_STATIC_IP
    IPADDR(&ulStaticIP,  192, 168, 1, 200);
    IPADDR(&ulSubnetMask, 255, 255, 255, 0);
    IPADDR(&ulGatewayIP, 192, 168, 1, 1);
    lwIPInit(pucMACArray, ulStaticIP, ulSubnetMask, ulGatewayIP, IPADDR_USE_STATIC);
#else
    /* Use DHCP */
    lwIPInit(pucMACArray, 0, 0, 0, IPADDR_USE_DHCP);
    /* Indicate that DHCP has started */
    UARTprintf("Waiting for IP\n");
#endif

    /* Initialize the TFTP server */
    TFTPFSInit();

    /*
       Set the interrupt priorities.  We set the SysTick interrupt to a higher
       priority than the Ethernet interrupt to ensure that the file system
       tick is processed if SysTick occurs while the Ethernet handler is being
       processed.  This is very likely since all the TCP/IP and TFTPD work is
       done in the context of the Ethernet interrupt.
    */
    IntPriorityGroupingSet(4);
    IntPrioritySet(INT_ETH, ETHERNET_INT_PRIORITY);
    IntPrioritySet(FAULT_SYSTICK, SYSTICK_INT_PRIORITY);

    /* Loop while waiting for interrupts */
    while(1) {}
}
