/*
  tftpd_fs.c - Filesystem functions for TFTP server using lwIP.
                by Micromint Support <support@micromint.com>    Aug-2009

  Copyright (c) 2008-2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include "inc/hw_types.h"
#include "utils/lwiplib.h"
#include "utils/ustdlib.h"
#include "utils/uartstdio.h"
#include "utils/tftp.h"
#include "lmi_fs.h"
#include "tftp_fs.h"

struct tftp_state {
  struct fs_file *handle;
};

/*
  Writes a block of data to the filesystem.  The position of the block
  is determined by the block number (ulBlockNum) and offset (ulDataRemaining)
  fields of the psTFTP structure passed and the total length of the data to
  write is in ulDataLength.
*/
static tTFTPError
TFTPFSPutData(tTFTPConnection *psTFTP)
{
    struct tftp_state *ts;
    unsigned long ulOffset, ulLength;

    /*
      Where does this block of data get written to?  We calculate this knowing
      the block number that we are given (TFTP blocks are fixed sized) and
      the offset into the block as provided in the ulDataRemaining field.
    */
    ulOffset = ((psTFTP->ulBlockNum - 1) * TFTP_BLOCK_SIZE) +
               psTFTP->ulDataRemaining;

    /* Write data to filesystem */
    ts = (struct tftp_state *)psTFTP->pucUser;
    fs_seek(ts->handle, ulOffset);
    ulLength = fs_write(ts->handle, (char *)psTFTP->pucData, psTFTP->ulDataLength);

    /* Did we read the expected number of bytes? */
    if(ulLength == psTFTP->ulDataLength)
    {
        /* Yes - tell the caller everything went fine. */
        return(TFTP_OK);
    }
    else
    {
        /*
          We wrote a different number of bytes than the requested count.
          Report an error to caller.
        */
        psTFTP->pcErrorString = "Filesystem write error.";
        return(TFTP_ERR_NOT_DEFINED);
    }
}

/*
  Read a block of data from the filesystem.
*/
static tTFTPError
TFTPFSGetData(tTFTPConnection *psTFTP)
{
    struct tftp_state *ts;
    unsigned long ulOffset, ulLength;

    /*
      Where does this block of data get read from?  This is calculated from
      the requested TFTP block number.
    */
    ulOffset = ((psTFTP->ulBlockNum - 1) * TFTP_BLOCK_SIZE);

    /* Read the data from the flash device into the supplied buffer. */
    ts = (struct tftp_state *)psTFTP->pucUser;
    fs_seek(ts->handle, ulOffset);
    ulLength = fs_read(ts->handle, (char *)psTFTP->pucData, psTFTP->ulDataLength);

    /* Did we read the expected number of bytes? */
    if(ulLength == psTFTP->ulDataLength)
    {
        /* Yes - tell the caller everything went fine. */
        return(TFTP_OK);
    }
    else
    {
        /*
          We read a different number of bytes than the requested count.
          Report an error to caller.
        */
        psTFTP->pcErrorString = "Filesystem read error.";
        return(TFTP_ERR_NOT_DEFINED);
    }
}

/*
  Signals that the TFTP connection is being closed down.
*/
static void
TFTPFSClose(tTFTPConnection *psTFTP)
{
    struct tftp_state *ts;

    /* Deallocate memory for state. */
    ts = (struct tftp_state *)psTFTP->pucUser;
    if (ts) {
      if(ts->handle) {
        fs_close(ts->handle);
        ts->handle = NULL;
      }
      mem_free(ts);
    }
}

/*
  Checks incoming TFTP request to determine if we want to handle it.

  \param psTFTP points to the TFTP connection instance structure for this
  request.
  \param bGet is \b true of the request is a GET (read) or \b false if it is
  a PUT (write).
  \param pucFileName points to a NULL terminated string containing the
  requested filename.
  \param tTFTPMode indicates the requested transfer mode, ASCII or binary.

  This function is called by the TFTP server whenever a new request is
  received.  It checks the passed filename to determine whether it is valid.
  If the filename is valid, the function sets fields \e ulDataRemaining, \e
  pfnGetData and \e pfnClose in the \e psTFTP structure for a GET request or
  fields \e pfnPutData, \e pfnClosebefore returning
  \e TFTP_OK to continue processing the request.

  \return Returns \e TFTP_OK if the request should be processed or any other
  TFTP error code otherwise.  In cases where \e TFTP_OK is not returned, the
  field psTFTP->pcErrorString may be set with an ASCII error string which
  will be returned to the TFTP client.
*/
static tTFTPError
TFTPFSRequest(tTFTPConnection *psTFTP, tBoolean bGet, char *pucFileName,
                tTFTPMode eMode)
{
    struct tftp_state *ts;
    struct fs_file *file;

    /* Allocate memory for the structure that holds the connection state */
    ts = (struct tftp_state *)mem_malloc(sizeof(struct tftp_state));
    if (ts == NULL) {
      /* Out of memory */
      psTFTP->pcErrorString = "Out of memory.";
      return(TFTP_ERR_NOT_DEFINED);
    }
	psTFTP->pucUser = (unsigned char *)ts;

    UARTprintf("Incoming TFTP request %s %s.\n", bGet ? "GET" : "PUT",
               pucFileName);

    /* Is the file available? */
    file = fs_open(pucFileName);
    if(file != NULL)
    {
        /*
          Set the appropriate callback functions depending upon the
          type of request received.
        */
        ts->handle = file;
        psTFTP->pfnClose = TFTPFSClose;

        /* Is this a GET or a PUT request? */
        if(bGet)
        {
            /*
              GET request - fill in the image size and the data transfer
              function pointer.
            */
            psTFTP->pfnGetData = TFTPFSGetData;
            psTFTP->ulDataRemaining = file->len;
        }
        else
        {
            /*
              PUT request - fill in the data transfer function pointer.
            */
            psTFTP->pfnPutData = TFTPFSPutData;
        }
    }
    else
    {
        /* The filename is not valid. */
        psTFTP->pcErrorString = "File not found.";
        return(TFTP_FILE_NOT_FOUND);
    }

    /* If we get here, all is well and the transfer can continue. */
    return(TFTP_OK);
}

/*
  Initializes the TFTP server supporting the DK-LM3S9B96 board.

  This function initializes the lwIP TFTP server and starts listening for
  incoming requests from clients.  It must be called after the network stack
  is initialized and after SSIFlashInit().
 
  \return None.
*/
void TFTPFSInit(void)
{
    /*
      Initialize the TFTP module and pass it our board-specific GET and PUT
      request handler function pointer.
    */
    TFTPInit(TFTPFSRequest);
}
