/*
  kpad_lcd.c - Tests keypad and character LCD interfaces
           by Micromint Support <support@micromint.com>    Jan-2009

  Copyright (c) 2008-2010 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "inc/hw_gpio.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "driverlib/adc.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/systick.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "keypad.h"
#include "lcd.h"

#define OUT_PORT                  GPIO_PORTE_BASE
#define OUT_PORT_SYSCTL           SYSCTL_PERIPH_GPIOE
#define OUT_PORT_MASK             0x0f

static unsigned long tickCount;
/* Set SysTick interrupt to twice per second (500 ms)  */
#define TICK_FREQ               2
/* Use a 30 second timeout */
#define TICK_TIMEOUT            (30 * TICK_FREQ)
/* Reset every hour */
#define TICK_LIMIT              (3600 * TICK_FREQ)

/*
  ! \addtogroup example_list
  ! <h1>CPLD ISA card(kpad_lcd)</h1>
  !
  ! Tests keypad interface
*/

const char strOn[] = "On ";
const char strOff[] = "Off ";
const unsigned long xPos[] = {4, 14, 4, 14};
const unsigned long yPos[] = {1, 1, 2, 2};

/*
  Handler for SysTick interrupt
*/
void SysTickHandler(void)
{
    unsigned long temp;
    char str[5];

    /* Handle event */
    tickCount %= TICK_LIMIT; /* Periodically reset tick count */

    /* Get temperature value */
    if (tickCount < TICK_TIMEOUT)
    {
        ADCProcessorTrigger(ADC_BASE,0);
        ADCSequenceDataGet(ADC_BASE, 0, &temp);
        usprintf(str, "%4d", temp);
        LcdWriteAt(7, 3, str);
    }

    tickCount++; /* Increment tick count */
}

/*
  Handler for keypad interrupt
*/
void KeypadIntHandler(void)
{
    static unsigned long lastKey;
    unsigned long key, pin, val;
    char *str;

    /* Clear keypad interrupt */
    KeypadIntClear();

    /* Display keypad character */
    key = KeypadRead();
    if ((key >= '0') && (key < '4'))
    {
        /* Toggle pin */
        key -= '0';
        pin = 1 << key;
        val = GPIOPinRead(OUT_PORT, pin) ^ pin;
        GPIOPinWrite(OUT_PORT, pin, val);
        str = (char *) (val != 0 ? strOn : strOff);
        UARTprintf("PE%d %s\n", key, str);
        LcdWriteAt(xPos[key], yPos[key], str);
    } else {
        UARTprintf("%c", key);
    }

    /* If ending sequence pressed, set timeout to end scan */
    tickCount = (key == 'F' && lastKey == 'F') ? TICK_TIMEOUT : 0;
    lastKey = key;
}

/*
  Main function to test keypad
*/
int main(void)
{
    /* If running on Rev A2 silicon, turn the LDO voltage up to 2.75V */
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

#if 1
    /* Set the processor to run at 50 MHz */
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);
#else
    /* Set the clock to run directly from the crystal at 8 MHz */
    SysCtlClockSet(SYSCTL_SYSDIV_1 | SYSCTL_USE_OSC | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);
#endif

    /* Enable peripherals and set GPIO pins */
    /* --- Enable UART --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);

    /* Configure SysTick to interrupt at TICK_FREQ frequency */
    SysTickPeriodSet(SysCtlClockGet()/TICK_FREQ);
    SysTickEnable();
    IntRegister(FAULT_SYSTICK, SysTickHandler);
    SysTickIntEnable();

    /* Configure ADC for temperature */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC);
    ADCSequenceConfigure(ADC_BASE,0,ADC_TRIGGER_PROCESSOR, 0);
    ADCSequenceStepConfigure(ADC_BASE, 0, 0,
                             ADC_CTL_IE | ADC_CTL_END | ADC_CTL_TS);
    ADCSequenceEnable(ADC_BASE, 0);

    /* --- Initialize keypad --- */
    KeypadInit();
    KeypadIntEnable(KeypadIntHandler);

    /* --- Initialize LCD controller --- */
    LcdInit();

    /* Enable PE0-3 for output */
    SysCtlPeripheralEnable(OUT_PORT_SYSCTL);
    GPIOPinTypeGPIOOutput(OUT_PORT, OUT_PORT_MASK);
    GPIOPinWrite(OUT_PORT, OUT_PORT_MASK, 0);

    /* Test keypad */
    UARTprintf("Keypad-LCD Demo\n");
    UARTprintf("Press 1 to 4 to\n");
    UARTprintf("toggle PE0 to PE3\n");
    UARTprintf("(\'F\' twice to end)\n");
    LcdWriteAt(0, 0, "Keypad-LCD Demo     ");
    LcdWriteAt(0, 1, "PE0 Off   PE1 Off   ");
    LcdWriteAt(0, 2, "PE2 Off   PE3 Off   ");
    LcdWriteAt(0, 3, "Temp:               ");

    /* Loop while waiting for interrupts */
    while(tickCount < TICK_TIMEOUT) {}

    /* Shutdown keypad to restore JTAG functionality */
    UARTprintf("\nKeypad-LCD demo done.\n");
    UARTprintf("Shutting down keypad.\n");
    UARTprintf("Press RESET button to restart...\n");
    LcdWriteAt(0, 0, "Keypad-LCD demo done");
    LcdWriteAt(0, 1, "Shutting down keypad");
    LcdWriteAt(0, 2, "Press RESET         ");
    LcdWriteAt(0, 3, "  to restart...     ");
    KeypadIntDisable();
    KeypadShut();

    /* Loop forever */
    while(1) {}
}
