/*
  keypad.c - Keypad functions for the Micromint Eagle 100(Cortex-M3)
                    by Micromint Support <support@micromint.com>    Aug-2008

  Uses PB0-PB7 for keypad I/O

  Copyright (c) 2008-2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "keypad.h"

static void KeypadSetPins(unsigned long);
static void delay_us(unsigned long);

#if 1
static unsigned long KeypadData[4][4] = { {'1','2','3','A'},
                                        {'4','5','6','B'},
                                        {'7','8','9','C'},
                                        {'0','F','E','D'}};
#else
static unsigned long KeypadData[4][4] = { {'0','1','2','3'},
                                        {'4','5','6','7'},
                                        {'8','9','A','B'},
                                        {'C','D','E','F'}};
#endif
static unsigned long Index[] = {0, 0, 1, 0, 2, 0, 0, 0, 3};

/* -------------------- Keypad functions -------------------- */
/* Initialize Keypad */
void KeypadInit(void)
{
    /* Disable JTAG be able to use PB7 */
    JtagDisable();
    /* Set rows as inputs and columns as outputs */
    KeypadSetPins(0);
}

/* Helper function to set keypad pins */
static void KeypadSetPins(unsigned long i)
{
    unsigned long InPins, OutPins;

    InPins = i ? KEYPAD_COL_PINS : KEYPAD_ROW_PINS;
    OutPins = i ? KEYPAD_ROW_PINS : KEYPAD_COL_PINS;

    /* Set inputs with pull-ups */
    GPIOPinTypeGPIOInput(KEYPAD_GPIO_BASE, InPins);
    GPIOPadConfigSet(KEYPAD_GPIO_BASE, InPins, GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD_WPU);
    /* Set outputs and set to low */
    GPIOPinTypeGPIOOutput(KEYPAD_GPIO_BASE, OutPins);
    GPIOPinWrite(KEYPAD_GPIO_BASE, OutPins, 0);
}

/* Shutdown Keypad */
void KeypadShut(void)
{
    /* Restore keypad columns as inputs */
    GPIOPinTypeGPIOInput(KEYPAD_GPIO_BASE, KEYPAD_COL_PINS);
    /* Enable JTAG */
    JtagEnable();
}

/* Read keypad character */
unsigned long KeypadRead(void)
{
    unsigned long row = 0, col = 0, key = '~';

    /* Disable interrupts */
    KeypadIntDisable();
    delay_us(1000); /* Delay for debounce */

    /* If any row pin is low, save as row value */
    row = GPIOPinRead(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS);
    row = (~row & KEYPAD_ROW_PINS) >> KEYPAD_ROW_SHIFT;
    if (row != 0)
    {
        /* Switch pin I/O to find column */
        KeypadSetPins(1);
        delay_us(1000);
        col = GPIOPinRead(KEYPAD_GPIO_BASE, KEYPAD_COL_PINS);
        col = (~col & KEYPAD_COL_PINS) >> KEYPAD_COL_SHIFT;
        key = KeypadData[Index[row]][Index[col]];

        /* Switch back */
        KeypadSetPins(0);
    }

    /* Wait for key up */
    while ((row = GPIOPinRead(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS))
              != KEYPAD_ROW_PINS) {}
    delay_us(1000);

    /* Enable interrupts */
    KeypadIntClear();
    GPIOPinIntEnable(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS);
    IntEnable(KEYPAD_INT_GPIO);
    return key;
}

/* Enable keypad interrupt */
void KeypadIntEnable(void (*pfnHandler)(void))
{
    /* Configure column pins for interrupt and enable interrupts */
    GPIOIntTypeSet(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS, GPIO_FALLING_EDGE);
    GPIOPinIntEnable(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS);
    IntRegister(KEYPAD_INT_GPIO, pfnHandler);
    IntEnable(KEYPAD_INT_GPIO);
}

/* Disable keypad interrupt */
void KeypadIntDisable(void)
{
    IntDisable(KEYPAD_INT_GPIO);
    GPIOPinIntDisable(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS);
}

/* Clear keypad interrupt */
void KeypadIntClear(void)
{
    /* Clear GPIO interrupt */
    GPIOPinIntClear(KEYPAD_GPIO_BASE, KEYPAD_ROW_PINS);
}

/* Approximate delay for a number of microseconds */
static void delay_us(unsigned long n)
{
    volatile unsigned long i;
    unsigned long iMax = n * 10;

    for (i=0; i<iMax; i++);
}
