/*
  rtc.c - Real-Time Clock Calendar functions for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    November-2009
*/
#include "rtc.h"

const unsigned char byte_rev_table[256] = {
         0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
         0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
         0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
         0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
         0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
         0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
         0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
         0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
         0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
         0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
         0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
         0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
         0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
         0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
         0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
         0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
         0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
         0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
         0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
         0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
         0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
         0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
         0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
         0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
         0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
         0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
         0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
         0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
         0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
         0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
         0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
         0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff,
};

static unsigned long hr2reg(int hour);
static unsigned long reg2hr(int hour);
/* -------------------- RTC functions -------------------- */

/* Initialize the RTC */
unsigned long RtcInit(void)
{
  unsigned char temp;
  SysCtlPeripheralEnable(RTC_PORT_SYSCTL);
  SysCtlPeripheralEnable(RTC_I2C_BUS_SYSCTL);
  GPIOPinTypeI2C(RTC_PORT, RTC_I2C_BUS_IO);
  I2CMasterInitExpClk(RTC_I2C_BUS, SysCtlClockGet(), false); //Enable 100kHz I2C Bus
  /* Read status register 1 */
  I2cRead(RTC_I2C_BUS, CMD_STATUS1, &temp, 1);
  /* Check to see if RTC was intialized before if so do not reinitialize it */
  if (!(temp & (FLAG_POC | FLAG_BLD)))
     return 0;
  /* Initialize the RTC to run in 24 hour mode */
  temp |= (FLAG_RESET | FLAG_24H);
  temp &= 0xf0;
  I2cWrite(RTC_I2C_BUS, CMD_STATUS1, &temp, 1);
  return 0;
}
/* Read Status Registers */
unsigned long RtcRdRegister(unsigned char *data, unsigned char reg)
{
  I2cRead(RTC_I2C_BUS, reg, data, 1);
  return 0;
}

/* Write Status Registers */
unsigned long RtcWrRegister(unsigned char *data, unsigned char reg)
{
  I2cWrite(RTC_I2C_BUS, reg, data, 1);
  return 0;
}
/* Set the date and time */
unsigned long RtcSetDateTime(struct rtc_time *tm)
{
  unsigned char buf[7];
  unsigned char i;

  buf[BYTE_SECS] = BIN2BCD(tm->tm_sec);
  buf[BYTE_MINS] = BIN2BCD(tm->tm_min);
  buf[BYTE_HOURS] = hr2reg(tm->tm_hour);
  buf[BYTE_WDAY] = BIN2BCD(tm->tm_wday);
  buf[BYTE_DAY] = BIN2BCD(tm->tm_mday);
  buf[BYTE_MONTH] = BIN2BCD(tm->tm_mon);
  buf[BYTE_YEAR] = BIN2BCD(tm->tm_year);
  for (i = 0; i < 7; ++i)
  {
    buf[i] = byte_rev_table[buf[i]];
  }
  /*Set the AM/PM flag. Does not matter if in 24 hour mode*/
  if (tm->tm_ampm) buf[BYTE_HOURS] |= FLAG_AMPM;
  else buf[BYTE_HOURS] &= ~FLAG_AMPM;
  I2cWrite(RTC_I2C_BUS, CMD_TIME1, &buf[0], 7);
  return 0;
}

/* Read the date and time */
unsigned long RtcReadDateTime(struct rtc_time *tm)
{
  unsigned char buf[7];
  unsigned char i;
  I2cRead(RTC_I2C_BUS, CMD_TIME1, &buf[0], 7);
  tm->tm_ampm = (buf[BYTE_HOURS] & FLAG_AMPM)>>1;

  for (i = 0; i < 7; ++i)
  {
    buf[i] = byte_rev_table[buf[i]];
  }
  tm->tm_sec = BCD2BIN(buf[BYTE_SECS]);
  tm->tm_min = BCD2BIN(buf[BYTE_MINS]);
  tm->tm_hour = reg2hr(buf[BYTE_HOURS]);
  tm->tm_wday = BCD2BIN(buf[BYTE_WDAY]);
  tm->tm_mday = BCD2BIN(buf[BYTE_DAY]);
  tm->tm_mon = BCD2BIN(buf[BYTE_MONTH]);
  tm->tm_year = BCD2BIN(buf[BYTE_YEAR]);
  return 0;
}

/* Change the time format from one format to the other (24 hour or 12 hour) */
unsigned long RtcChngTimeFormat(void)
{
  unsigned char cmd_status1;
  I2cRead(RTC_I2C_BUS, CMD_STATUS1, &cmd_status1, 1);
  if (cmd_status1 && FLAG_24H)
  {
    cmd_status1  &= ~FLAG_24H;
    I2cWrite(RTC_I2C_BUS, CMD_STATUS1, &cmd_status1, 1);
    return 12;
  }
  else
  {
    cmd_status1  |= FLAG_24H;
    I2cWrite(RTC_I2C_BUS, CMD_STATUS1, &cmd_status1, 1);
    return 24;
  }
}

/* Return what the time format is set to */
unsigned long RtcTimeFormat(void)
{
  unsigned char cmd_status1;
  I2cRead(RTC_I2C_BUS, CMD_STATUS1, &cmd_status1, 1);
  if (cmd_status1 && FLAG_24H)
  {
    return 24;
  }
  else
  {
    return 12;
  }
}

/* Set the alarm time */
unsigned long RtcSetAlTime(unsigned long occur, struct rtc_time *altm)
{
  unsigned char buf[3];
  unsigned char i;
  buf[BYTE_ALWDAY] = BIN2BCD(altm->tm_wday);
  buf[BYTE_ALHOUR] = hr2reg(altm->tm_hour);
  buf[BYTE_ALMIN] = BIN2BCD(altm->tm_min);

  for (i = 0; i < 3; ++i)
  {
    buf[i] = byte_rev_table[buf[i]];
  }
  switch (occur)
  {
    case AL_WEEKLY: buf[BYTE_ALWDAY] |= FLAG_AE;
                    buf[BYTE_ALHOUR] |= FLAG_AE;
                    buf[BYTE_ALMIN] |= FLAG_AE;
                    break;
    case AL_DAILY:  buf[BYTE_ALWDAY] &= ~FLAG_AE;
                    buf[BYTE_ALHOUR] |= FLAG_AE;
                    buf[BYTE_ALMIN]  |= FLAG_AE;
                    break;
    case AL_HOURLY: buf[BYTE_ALWDAY] &= ~FLAG_AE;
                    buf[BYTE_ALHOUR] &= ~FLAG_AE;
                    buf[BYTE_ALMIN] |= FLAG_AE;
                    break;
    default:        return 1; /* return with an error */
  }
  /*Set the AM/PM flag. Does not matter if in 24 hour mode*/
  if (altm->tm_ampm) buf[BYTE_ALHOUR] |= FLAG_AMPM;
  else buf[BYTE_ALHOUR] &= ~FLAG_AMPM;
  I2cWrite(RTC_I2C_BUS, CMD_INT1, &buf[0], 3);
  return 0;
}

/* Read the alarm time */
unsigned long RtcReadAlTime(struct rtc_time *altm)
{
  unsigned char buf[3];
  unsigned char i;
  unsigned long occurrance;
  I2cRead(RTC_I2C_BUS, CMD_INT1, &buf[0], 3);
  altm->tm_ampm = (buf[BYTE_ALHOUR] & FLAG_AMPM)>>1;
  /* Check to see what alarm flags are enabled */
  if ((buf[BYTE_ALWDAY] & FLAG_AE) &&
      (buf[BYTE_ALHOUR] & FLAG_AE) &&
      (buf[BYTE_ALMIN] & FLAG_AE))
  {
    occurrance = AL_WEEKLY;
  }
  else if (!(buf[BYTE_ALWDAY] & FLAG_AE) &&
           (buf[BYTE_ALHOUR] & FLAG_AE) &&
           (buf[BYTE_ALMIN] & FLAG_AE))
  {
    occurrance = AL_DAILY;
  }
  else if (!(buf[BYTE_ALWDAY] & FLAG_AE) &&
           !(buf[BYTE_ALHOUR] & FLAG_AE) &&
           (buf[BYTE_ALMIN] & FLAG_AE))
  {
    occurrance = AL_HOURLY;
  }
  else
  {
    return(3); /* Return error */
  }

  for (i = 0; i < 3; ++i)
  {
    buf[i] &= ~FLAG_AE;  /* Remove the alarm enable bit */
    buf[i] = byte_rev_table[buf[i]];
  }
  altm->tm_min = BCD2BIN(buf[BYTE_ALMIN]);
  altm->tm_hour = reg2hr(buf[BYTE_ALHOUR]);
  altm->tm_wday = BCD2BIN(buf[BYTE_ALWDAY]);
  return occurrance;
}

/* Enable the alarm */
unsigned long RtcEnableAlarm(unsigned long mode)
{
  unsigned char cmd_status2;
  I2cRead(RTC_I2C_BUS, CMD_STATUS2, &cmd_status2, 1);
  /* Enable the alarm on the RTC */
  if (mode == AL_PERMIN)
  {
    cmd_status2 |= FLAG_INT1ME;
    cmd_status2 &= ~(FLAG_INT1AE | FLAG_INT1FE);
  }
  else if((mode == AL_WEEKLY) || (mode == AL_DAILY) || (mode == AL_HOURLY))
  {
    cmd_status2 |= FLAG_INT1AE;
    cmd_status2 &= ~(FLAG_INT1ME | FLAG_INT1FE);
  }
  I2cWrite(RTC_I2C_BUS, CMD_STATUS2, &cmd_status2, 1);
  return 0;
}

/* Disable the alarm */
unsigned long RtcDisableAlarm(void)
{
  unsigned char cmd_status2;
  I2cRead(RTC_I2C_BUS, CMD_STATUS2, &cmd_status2, 1);
  /* Disable the alarm on the RTC */
  cmd_status2 &= ~(FLAG_INT1AE| FLAG_INT1ME | FLAG_INT1FE);
  I2cWrite(RTC_I2C_BUS, CMD_STATUS2, &cmd_status2, 1);
  return 0;
}

/* Format the hour when writing the time */
unsigned long hr2reg(int hour)
{
  unsigned char cmd_status1;
  I2cRead(RTC_I2C_BUS, CMD_STATUS1, &cmd_status1, 1);
  if (cmd_status1 && FLAG_24H)
  {
     return BIN2BCD(hour);
  }
  if (hour < 12) return BIN2BCD(hour);
  return 0x40 | BIN2BCD(hour - 12);
}

/* Format the hour when reading the time */
unsigned long reg2hr(int hour)
{
  unsigned char cmd_status1;
  hour = BCD2BIN(hour & 0x3f);
  I2cRead(RTC_I2C_BUS, CMD_STATUS1, &cmd_status1, 1);
  if (cmd_status1 && FLAG_24H) return hour;
  if (hour == 0x00) hour += 12;
  return hour;
}
