/*
  xio.c - Extended I/O functions for the Micromint Eagle SBCs (Cortex-M3)
          by Micromint Support <support@micromint.com>    Jul-2008

  Uses SSI0 as SSI master to manage the Extended I/O ports on CPLD

  Copyright (c) 2008-2010 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "xio.h"

#if !defined(NO_XIO)

static unsigned long XioPut(unsigned long);

/* -------------------- Extended I/O functions -------------------- */
/* Initialize extended I/O */
void XioInit(void)
{
    /* Initialize SSI port */
    SysCtlPeripheralEnable(XIO_SSI_SYSCTL_PERIPH);
    SysCtlPeripheralEnable(XIO_GPIO_SYSCTL_PERIPH);
    SysCtlPeripheralEnable(XIO_CS_GPIO_SYSCTL_PERIPH);
    GPIOPinTypeSSI(XIO_GPIO_PORT_BASE, XIO_SSI_PINS);
    GPIOPinTypeGPIOOutput(XIO_CS_GPIO_PORT_BASE, XIO_CS);
    GPIOPadConfigSet(XIO_GPIO_PORT_BASE, XIO_SSI_PINS, GPIO_STRENGTH_4MA,
                     GPIO_PIN_TYPE_STD_WPU);
    GPIOPadConfigSet(XIO_CS_GPIO_PORT_BASE, XIO_CS, GPIO_STRENGTH_4MA,
                     GPIO_PIN_TYPE_STD_WPU);
    GPIOPinWrite(XIO_CS_GPIO_PORT_BASE, XIO_CS, XIO_CS); /* Deselect extended I/O */
    /* Freescale SPI frame format mode 0 (polarity=0, phase=0)
         SSI master, 4 MHz clock, 9 data bits
    */
    SSIConfigSetExpClk(XIO_SSI_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_0,
                       SSI_MODE_MASTER, 8000000, 9);
    SSIEnable(XIO_SSI_BASE);
}

/* Helper function for writing data to SSI port */
static unsigned long XioPut(unsigned long data)
{
    unsigned long rcvData;

    GPIOPinWrite(XIO_CS_GPIO_PORT_BASE, XIO_CS, 0); /* Select extended I/O */
    SSIDataPut(XIO_SSI_BASE, data); /* Send data */
    SSIDataGet(XIO_SSI_BASE, &rcvData); /* Flush received data */
    GPIOPinWrite(XIO_CS_GPIO_PORT_BASE, XIO_CS, XIO_CS); /* Deselect extended I/O */

    return rcvData;
}

/* Write a byte to an extended I/O port */
unsigned long XioWrite(unsigned long port, unsigned long data)
{
    XioPut(XIO_CMD_SEL | XIO_CMD_WRITE | port); /* Send write command */
    return XioPut(data & XIO_DATA_MASK); /* Send data */
}

/* Write a block of data to an extended I/O port */
unsigned long XioWriteBlock(unsigned long port, unsigned long* ptr, unsigned long count)
{
    unsigned long i = count;

    XioPut(XIO_CMD_SEL | XIO_CMD_WRITE | port); /* Send write command */
    while (i--)
    {
      XioPut(*ptr++ & XIO_DATA_MASK); /* Send data */
    }

    return count;
}

/* Read a byte from an extended I/O port */
unsigned long XioRead(unsigned long port)
{
    XioPut(XIO_CMD_SEL | XIO_CMD_READ | port); /* Send read command */
    return (XioPut(0xff) & XIO_DATA_MASK); /* Send dummy data & return value */
}

/* Read a block of data into a buffer */
unsigned long XioReadBlock(unsigned long port, unsigned long* ptr, unsigned long count)
{
    unsigned long i = count;

    while (i--)
      *ptr++ = XioRead(port);

    return count;
}

/* -------------------- ISA bus functions -------------------- */
/* Initialize PC/104 controller */
void IsaInit(void)
{
    XioInit();
}

/* Write data at a specified address */
unsigned long IsaWrite(unsigned long addr, unsigned long data)
{
    unsigned long rcvData;

    XioWrite(ISA_ADDRL, (addr & ISA_DATA_MASK)); /* Write low address */
    XioWrite(ISA_ADDRH, ((addr >> 8) & ISA_DATA_MASK)); /* Write high address */
    rcvData = XioWrite(ISA_PORT, (data & ISA_DATA_MASK)); /* Write data */
    XioRead(ISA_STATUS); /* Read status to complete cycle */
    rcvData = XioWrite(ISA_PORT, (data & ISA_DATA_MASK));
    XioRead(ISA_STATUS);
	return rcvData;
}

unsigned long IsaWriteBlock(unsigned long addr, unsigned long* ptr, unsigned long count)
{
    unsigned long i = count;

    XioWrite(ISA_ADDRL, (addr & ISA_DATA_MASK)); /* Write low address */
    XioWrite(ISA_ADDRH, ((addr >> 8) & ISA_DATA_MASK)); /* Write high address */

    /* Write data block */
    while (i--)
	{
      XioWrite(ISA_PORT, (*ptr & ISA_DATA_MASK));
      XioRead(ISA_STATUS);
      XioWrite(ISA_PORT, (*ptr++ & ISA_DATA_MASK)); /* Write data */
      XioRead(ISA_STATUS); /* Read status to complete cycle */
	}

    return count;
}

/* Read data from a specified address */
unsigned long IsaRead(unsigned long addr)
{
    XioWrite(ISA_ADDRL, (addr & ISA_DATA_MASK)); /* Write low address */
    XioWrite(ISA_ADDRH, ((addr >> 8) & ISA_DATA_MASK)); /* Write high address */
    XioRead(ISA_LATCH); /* Latch data */
    XioRead(ISA_STATUS); /* Read status to complete cycle */
    return XioRead(ISA_PORT); /* Read data */
}

unsigned long IsaReadBlock(unsigned long addr, unsigned long* ptr, unsigned long count)
{
    unsigned long i = count;

    XioWrite(ISA_ADDRL, (addr & ISA_DATA_MASK)); /* Write low address */
    XioWrite(ISA_ADDRH, ((addr >> 8) & ISA_DATA_MASK)); /* Write high address */

    /* Read data block */
    while (i--)
	{
      XioRead(ISA_LATCH); /* Latch data */
      XioRead(ISA_STATUS); /* Read status to complete cycle */
      *ptr++ = XioRead(ISA_PORT);
	}

    return count;
}

#endif /* NO_XIO */
